<?php

namespace App\Http\Controllers;

use App\Category;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Validator;
use Image;
use File;


class Categories extends Controller
{
    /** Created by: Sanjaya Prasanna
     * summery: Display all existing categories and category create form.
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index()
    {

        if (Auth::user()->role > 0){
            $data = Category::all();
            $title = "Manage Category";
            return view('admin.category.manage', compact('data','title'));
        } else {
            redirect('/login')->with('status', 'You do not have permission for access this page');
        }


    }

    /** Created by: Sanjaya Prasanna
     * summery: Insert category data to database.
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|max:100|unique:categories',
            'feature_image' => 'required|image|mimes:jpeg,png,jpg,gif|max:1024',
            'status' => 'required',
            'display_order' => 'required',
        ],
            [
                'name.required' => 'Category Name field is required',
                'feature_image.required' => 'Feature image is required',
                'name.unique' => 'This category name is already exists',
                'status.required' => 'Category Status field is required',
                'display_order.required' => 'Category Display Order field is required',
            ]
        );
        if ($validator->fails()) {
            return redirect('admin/category')
                ->withErrors($validator)
                ->withInput();
        }

        $image= $request->file('feature_image');
        $image_path = Image::make($image);
        $set_img_path = time().$image->getClientOriginalName();
        $image_path->save( public_path().'/uploads/category/'.$set_img_path);

        $title = strtolower($request->name);
        Category::create([
            'name' => $request->name,
            'slug' => str_slug($title, '-'),
            'feature_image' => $set_img_path,
            'status' => $request->status,
            'display_order' => $request->display_order,
            'user_id' => Auth::user()->id,
        ]);
        return back()->with('status', 'Category Successfully created');
    }

    /** Created by: Sanjaya Prasanna
     * summery: Category edit view.
     * @param $id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit($id)
    {

        if (Auth::user()->role > 0){
            $data = Category::all()->find($id);
            $title = "Edit Category";
            return view('admin.category.edit', compact('data', 'title'));
        } else {
            redirect('/login')->with('status', 'You do not have permission for access this page');
        }
    }

    /** Created by: Sanjaya Prasanna
     * summery: Update category data to database.
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function update(Request $request, $id){

        $validator = Validator::make($request->all(), [
            'name' => 'required|max:100',
            'status' => 'required',
            'display_order' => 'required',
        ],
            [
                'name.required' => 'Category Name field is required',
                'status.required' => 'Category Status field is required',
                'display_order.required' => 'Category Display Order field is required',
            ]
        );
        if ($validator->fails()) {
            return redirect('admin/category/edit/'.$id)
                ->withErrors($validator)
                ->withInput();
        }
        $title = strtolower($request->name);
        $data = Category::all()->find($id);
        $data['name'] = $request->name;
        $data['slug'] =  str_slug($title, '-');
        $data['status'] = $request->status;
        $data['display_order'] = $request->display_order;
        $data['user_id'] = Auth::user()->id;
        $data->save();

        return redirect('admin/category')->with('status', 'Category successfully updated');

    }

    /** Created by: Sanjaya Prasanna
     * summery: Update category feature image url in database and
     *          delete old image from directory, upload new image to directory.
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update_feature_image(Request $request, $id)
    {

        File::delete('uploads/category/'.$request->current_image);

        $image= $request->file('feature_image');
        $image_path = Image::make($image);
        $set_img_path = time().$image->getClientOriginalName();
        $image_path->save( public_path().'/uploads/category/'.$set_img_path);

        $data = Category::all()->find($id);
        $data['feature_image'] = $set_img_path;
        $data->save();
        return back()->with('status', 'Category feature image successfully updated');

    }

    /** Created by: Sanjaya Prasanna
     * summery: Delete category form database and delete category relate image form directory.
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function destroy(Request $request, $id)
    {
        $data = Category::findOrFail($id);
        File::delete('uploads/category/'.$data->feature_image);
        $data->delete();
        return redirect('admin/category')->with('status', 'Category successfully Deleted');

    }
}
